defmodule Lighthouse.Client do
  use GenServer
  require Logger

  def start_link(opts) do
    GenServer.start_link(__MODULE__, opts)
  end

  def init(opts) do
    send(self(), :connect)
    {:ok, Map.merge(%{}, opts)}
  end

  def handle_info(:connect, state) do
    Logger.info("Lighthouse attempting connection")

    Riptide.Client.start_link(
      state.url,
      [
        Lighthouse.Commands.Console
      ],
      name: Lighthouse
    )
    |> case do
      {:ok, pid} ->
        Logger.info("Lighthouse connected")

        {:ok, _} =
          Riptide.Client.call(
            Lighthouse,
            "auth.node",
            %{
              token: state.token,
              node: state[:name] || :inet.gethostname() |> elem(1) |> String.Chars.to_string()
            }
          )

        Logger.info("Lighthouse authenticated")
        :ok

      _ ->
        Process.send_after(self(), :connect, :timer.seconds(5))
    end

    {:noreply, state}
  end
end
